@@ -1,11 +1,9 @@
 package Entities::Backend::Memory;
-BEGIN {
-  $Entities::Backend::Memory::VERSION = '0.2';
-}
 
-use Moose;
-use namespace::autoclean;
 use Carp;
+use Moo;
+use MooX::Types::MooseLike::Base qw/ArrayRef/;
+use namespace::autoclean;
 
 with 'Entities::Backend';
 
@@ -17,7 +15,7 @@ Entities::Backend::Memory - A simple backend that stores all data in memory, for
 
 =head1 VERSION
 
-version 0.2
+version 0.3
 
 =head1 SYNOPSIS
 
@@ -49,7 +47,10 @@ objects is provided, it will replace the current list.
 
 =cut
 
-has 'roles' => (is => 'rw', isa => 'ArrayRef[Entities::Role]');
+has 'roles' => (
+	is => 'rw',
+	isa => ArrayRef
+);
 
 =head2 users( [\@users] )
 
@@ -59,7 +60,10 @@ objects is provided, it will replace the current list.
 
 =cut
 
-has 'users' => (is => 'rw', isa => 'ArrayRef[Entities::User]');
+has 'users' => (
+	is => 'rw',
+	isa => ArrayRef
+);
 
 =head2 actions( [\@actions] )
 
@@ -69,7 +73,10 @@ objects is provided, it will replace the current list.
 
 =cut
 
-has 'actions' => (is => 'rw', isa => 'ArrayRef[Entities::Action]');
+has 'actions' => (
+	is => 'rw',
+	isa => ArrayRef
+);
 
 =head2 plans( [\@plans] )
 
@@ -79,7 +86,10 @@ objects is provided, it will replace the current list.
 
 =cut
 
-has 'plans' => (is => 'rw', isa => 'ArrayRef[Entities::Plan]');
+has 'plans' => (
+	is => 'rw',
+	isa => ArrayRef
+);
 
 =head2 customers( [\@customers] )
 
@@ -89,7 +99,10 @@ objects is provided, it will replace the current list.
 
 =cut
 
-has 'customers' => (is => 'rw', isa => 'ArrayRef[Entities::Customer]');
+has 'customers' => (
+	is => 'rw',
+	isa => ArrayRef
+);
 
 =head2 features( [\@features] )
 
@@ -99,7 +112,10 @@ objects is provided, it will replace the current list.
 
 =cut
 
-has 'features' => (is => 'rw', isa => 'ArrayRef[Entities::Feature]');
+has 'features' => (
+	is => 'rw',
+	isa => ArrayRef
+);
 
 =head1 METHODS IMPLEMENTED
 
@@ -300,7 +316,7 @@ L<http://search.cpan.org/dist/Entities/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010 Ido Perlmuter.
+Copyright 2010-2013 Ido Perlmuter.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
@@ -310,5 +326,4 @@ See http://dev.perl.org/licenses/ for more information.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-1;
\ No newline at end of file
+1;
