@@ -1,13 +1,10 @@
 package Entities::Role;
-BEGIN {
-  $Entities::Role::VERSION = '0.2';
-}
 
-use Moose;
-use Moose::Util::TypeConstraints;
-use MooseX::Types::DateTime;
-use namespace::autoclean;
 use Carp;
+use Moo;
+use MooX::Types::MooseLike::Base qw/Any Str Bool ArrayRef/;
+use Scalar::Util qw/blessed/;
+use namespace::autoclean;
 
 # ABSTRACT: A collection of possibly related actions granted to users.
 
@@ -17,7 +14,7 @@ Entities::Role - A collection of possibly related actions granted to users.
 
 =head1 VERSION
 
-version 0.2
+version 0.3
 
 =head1 SYNOPSIS
 
@@ -61,7 +58,12 @@ internally.
 
 =cut
 
-has 'id' => (is => 'ro', isa => 'Any', predicate => 'has_id', writer => '_set_id');
+has 'id' => (
+	is => 'ro',
+	isa => Any,
+	predicate => 'has_id',
+	writer => '_set_id'
+);
 
 =head2 name()
 
@@ -69,7 +71,11 @@ Returns the name of the role.
 
 =cut
 
-has 'name' => (is => 'ro', isa => 'Str', required => 1);
+has 'name' => (
+	is => 'ro',
+	isa => Str,
+	required => 1
+);
 
 =head2 description()
 
@@ -81,9 +87,13 @@ Changes the description text of the object to the provided text.
 
 =cut
 
-has 'description' => (is => 'ro', isa => 'Str', writer => 'set_description');
+has 'description' => (
+	is => 'ro',
+	isa => Str,
+	writer => 'set_description'
+);
 
-=head2 _roles( [\@roles] )
+=head2 roles( [\@roles] )
 
 In scalar context, returns an array-ref of all role names this role
 inherits from. In list context returns an array. If an array-ref of
@@ -95,24 +105,11 @@ Returns a true value if the role inherits from any other roles.
 
 =cut
 
-has '_roles' => (is => 'rw', isa => 'ArrayRef[Str]', predicate => 'has_roles');
-
-=head2 roles()
-
-Returns an array of all role objects this role inherits from.
-
-=cut
-
-sub roles {
-	my $self = shift;
-
-	my @roles;
-	foreach ($self->_roles) {
-		push(@roles, $self->parent->get_role($_));
-	}
-
-	return @roles;
-}
+has 'roles' => (
+	is => 'rw',
+	isa => ArrayRef[Str],
+	predicate => 'has_roles'
+);
 
 =head2 _actions( [\@actions] )
 
@@ -124,26 +121,17 @@ action names is provided, it will replace the current list.
 
 Returns a true value if the role has been granted any actions.
 
-=cut
-
-has '_actions' => (is => 'rw', isa => 'ArrayRef[Str]', predicate => 'has_actions');
-
 =head2 actions()
 
-Returns an array of all action objects this role has been granted.
+Returns an array of all action names this role has been granted.
 
 =cut
 
-sub actions {
-	my $self = shift;
-
-	my @actions;
-	foreach ($self->_actions) {
-		push(@actions, $self->parent->get_action($_));
-	}
-
-	return @actions;
-}
+has 'actions' => (
+	is => 'rw',
+	isa => ArrayRef[Str],
+	predicate => 'has_actions'
+);
 
 =head2 is_super()
 
@@ -152,7 +140,11 @@ can do every possible action, in ANY SCOPE.
 
 =cut
 
-has 'is_super' => (is => 'ro', isa => 'Bool', default => 0);
+has 'is_super' => (
+	is => 'ro',
+	isa => Bool,
+	default => 0
+);
 
 =head2 created()
 
@@ -160,7 +152,11 @@ Returns a L<DateTime> object in the time the role object has been created.
 
 =cut
 
-has 'created' => (is => 'ro', isa => 'DateTime', default => sub { DateTime->now() });
+has 'created' => (
+	is => 'ro',
+	isa => sub { croak 'created must be a DateTime object' unless blessed $_[0] && blessed $_[0] eq 'DateTime' },
+	default => sub { DateTime->now() }
+);
 
 =head2 modified( [$dt] )
 
@@ -169,7 +165,11 @@ If a DateTime object is provided, it is set as the new modified value.
 
 =cut
 
-has 'modified' => (is => 'rw', isa => 'DateTime', default => sub { DateTime->now() });
+has 'modified' => (
+	is => 'rw',
+	isa => sub { croak 'modified must be a DateTime object' unless blessed $_[0] && blessed $_[0] eq 'DateTime' },
+	default => sub { DateTime->now() }
+);
 
 =head2 parent()
 
@@ -177,7 +177,11 @@ Returns the L<Entities::Backend> instance that stores this object.
 
 =cut
 
-has 'parent' => (is => 'ro', does => 'Entities::Backend', weak_ref => 1);
+has 'parent' => (
+	is => 'ro',
+	isa => sub { croak 'parent must be an Entities::Backend' unless blessed $_[0] && $_[0]->does('Entities::Backend') },
+	weak_ref => 1
+);
 
 with 'Abilities';
 
@@ -196,7 +200,7 @@ sub has_direct_action {
 		return;
 	}
 
-	foreach ($self->_actions) {
+	foreach ($self->actions) {
 		return 1 if $_ eq $action_name;
 	}
 
@@ -228,9 +232,9 @@ sub grant_action {
 	croak "Action $action_name does not exist." unless $action;
 
 	# add this action
-	my @actions = $self->_actions;
+	my @actions = $self->actions;
 	push(@actions, $action_name);
-	$self->_actions(\@actions);
+	$self->actions(\@actions);
 
 	return $self;
 }
@@ -259,11 +263,11 @@ sub drop_action {
 
 	# remove the action
 	my @actions;
-	foreach ($self->_actions) {
+	foreach ($self->actions) {
 		next if $_ eq $action_name;
 		push(@actions, $_);
 	}
-	$self->_actions(\@actions);
+	$self->actions(\@actions);
 
 	return $self;
 }
@@ -284,19 +288,19 @@ sub inherit_from_role {
 	croak "You must provide a role name." unless $role_name;
 
 	# do we already take from this role?
-	if ($self->takes_from($role_name)) {
+	if ($self->assigned_role($role_name)) {
 		carp "Role ".$self->name." already inherits from ".$role_name;
 		return $self;
 	}
 
 	# find this role, does it even exist?
-	my $role = $self->parent->get_role($role_name);
+	my $role = $self->get_role($role_name);
 	croak "Role $role_name does not exist." unless $role;
 
 	# add this role
-	my @roles = $self->_roles;
+	my @roles = $self->roles;
 	push(@roles, $role_name);
-	$self->_roles(\@roles);
+	$self->roles(\@roles);
 
 	return $self;
 }
@@ -315,50 +319,44 @@ sub dont_inherit_from_role {
 	croak "You must provide a role name." unless $role_name;
 
 	# does the user even have this role?
-	unless ($self->takes_from($role_name)) {
+	unless ($self->assigned_role($role_name)) {
 		carp "Role ".$self->name." doesn't inherit from role $role_name.";
 		return $self;
 	}
 
 	# remove the role
 	my @roles;
-	foreach ($self->_roles) {
+	foreach ($self->roles) {
 		next if $_ eq $role_name;
 		push(@roles, $_);
 	}
-	$self->_roles(\@roles);
+	$self->roles(\@roles);
 
 	return $self;
 }
 
-=head1 METHODS CONSUMED FROM Abilities
+=head2 get_role( $role_name )
 
-The following methods are consumed by this class from the L<Abilities>
-Moose role. See the documentation for that role for more information on
-these methods.
+Returns the role object of the role named C<$role_name>.
 
-=head2 can_perform( $action_name | @action_names )
+=cut
 
-=head2 takes_from( $role_name | @role_names )
-
-=head2 inherits_from_role( $role_name | @role_names )
-
-=head2 all_abilities()
+sub get_role { shift->parent->get_role(@_) }
 
 =head1 METHOD MODIFIERS
 
 The following list documents any method modifications performed through
 the magic of L<Moose>.
 
-=head2 around qw/_roles _actions/
+=head2 around qw/roles actions/
 
-If the C<_roles()> and C<_actions()> methods are called with no arguments
+If the C<roles()> and C<actions()> methods are called with no arguments
 and in list context - will automatically dereference the array-ref into
 arrays.
 
 =cut
 
-around qw/_roles _actions/ => sub {
+around qw/roles actions/ => sub {
 	my ($orig, $self) = (shift, shift);
 
 	if (scalar @_) {
@@ -427,7 +425,7 @@ L<http://search.cpan.org/dist/Entities/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010 Ido Perlmuter.
+Copyright 2010-2013 Ido Perlmuter.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
@@ -437,5 +435,4 @@ See http://dev.perl.org/licenses/ for more information.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-1;
\ No newline at end of file
+1;
